'' ----------------------------------------------------------------------------
'' Irrlicht Wrapper for Imperative Languages - Freebasic Examples
'' Frank Dodd (2006)
'' ----------------------------------------------------------------------------
'' Example 49 : Collision in a Loaded IrrEdit Scene
'' This example demonstrates collision in a scene that was created in IrrEdit
'' the nodes in the scene are found and then collision selection objects are
'' created for them in the same way as you would with a custom created scene
'' ----------------------------------------------------------------------------

'' ////////////////////////////////////////////////////////////////////////////
'' Includes for extension libraries
#include "IrrlichtWrapper.bi"

'' ////////////////////////////////////////////////////////////////////////////
'' global variables

' irrlicht objects
DIM Camera as irr_camera
DIM CameraNode as irr_node
DIM NodeGround as irr_node
DIM NodeBox as irr_node
DIM SelectorGround as irr_selector
DIM SelectorBox as irr_selector
DIM CombinedCollision as irr_selector
DIM CollisionAnimator as irr_animator


'' ////////////////////////////////////////////////////////////////////////////
'' GDB debugger main() function

' -----------------------------------------------------------------------------
' start the irrlicht interface
IrrStart( IRR_EDT_OPENGL, 640,480, 32, IRR_WINDOWED, _
          IRR_SHADOWS, IRR_IGNORE_EVENTS, IRR_VERTICAL_SYNC_ON )

' send the window caption
IrrSetWindowCaption( "Example 49: Collision in a Loaded Scene" )

' load an example scene created in irrEdit
IrrLoadScene( "./media/CollisionScene.irr" )

' we add a first person perspective camera to the scene so you can look about
' and move it into the center of the map
Camera = IrrAddFPSCamera
CameraNode = Camera
IrrSetNodePosition( CameraNode, 200, 100, 0 )
IrrSetNodeRotation( CameraNode, 0, -90, 0 )

' first we need to get references to the objects in the scene that the viewer
' can collide with
NodeGround = IrrGetSceneNodeFromName( "Ground" )
NodeBox = IrrGetSceneNodeFromName( "Pillar" )

' next we need to create collision objects from the nodes 
SelectorGround = IrrGetCollisionGroupFromBox( NodeGround )
SelectorBox = IrrGetCollisionGroupFromBox( NodeBox )

' now that we have collision objects for each of the nodes we need to combine
' them into a collision group
CombinedCollision = IrrCreateCombinedCollisionGroup ()

' creates a meta-selector that is a group of selector objects
IrrAddCollisionGroupToCombination ( CombinedCollision, SelectorGround )
IrrAddCollisionGroupToCombination ( CombinedCollision, SelectorBox )

' finally we add a collision animator to the camera, this constrains the camera
' so that its movement is restricted by collision with the collision groups we
' just created. the parameters we supply are as follows: -
'       The collision objects, the node to be effected by collision,
'       x, y and z radius of the collision area,
'       x, y and z force of gravity to apply to the node affected by collision,
'       x, y and z offset of the node
CollisionAnimator = IrrAddCollisionAnimator(_
        CombinedCollision, CameraNode,_
        30.0,30.0,30.0,_
        0.0,-3.0,0.0,_
        0.0,50.0,0.0 )

' we also hide the mouse pointer to see the view better
IrrHideMouse

' -----------------------------------------------------------------------------
' while the irrlicht environment is still running
WHILE IrrRunning
    ' begin the scene, erasing the canvas with sky-blue before rendering
    IrrBeginScene( 200, 200, 255 )

    ' draw the scene
    IrrDrawScene

    ' end drawing the scene and render it
    IrrEndScene
WEND

' -----------------------------------------------------------------------------
' Stop the irrlicht engine and release resources
IrrStop
